<?
  # Steganografia. Ukrywanie informacji tekstowych w plikach graficznych
  # (C) 2006 Piotr Gapiski
  # Kod dostpny na licencji LGPL

class Steganography
{
	function encode($text, $key, $image)
	{
		# zakoduj dugo wiadomoci w pierwszym pixelu
		if (empty($text) || empty($key)) return false;
		$text_len = strlen($text);
		$pixel = 1;
		$r = $text_len >> 2;
		$g = ($text_len - ($r << 2)) >> 1;
		$b = $text_len - ($r << 2) - ($g << 1);
		$color = $image->pixel_color($r, $g, $b);
		$image->set_pixel($pixel, $color);

		$key_len = strlen($key);
		$key_index = $color_index = $i = 0;

		while ($i < $text_len)
		{
			# znajd nastpny piksel
			$pixel += ord($key[$key_index]);
			$rgb = $image->get_pixel($pixel);

			# zakoduj bajt danych prywatnym kluczem
			$crypted_value = ord($text[$i] ^ $key[$key_index]);
			if ($image->greyscale) {
				$color = $image->pixel_color($crypted_value, $crypted_value, $crypted_value);
			}
			else {
				$color = $image->change_color_component($rgb, $color_index, $crypted_value);
			}
			$image->set_pixel($pixel, $color);
			
			$i += 1;
			$key_index = ($i % $key_len);
			$color_index = ($i % 3);
		}
		return true;
	}

	function decode($key, $image)
	{
		# dugo wiadomoci jest zakodowana w pierwszym pixelu
		$pixel = 1;
		$rgb = $image->get_pixel($pixel);
		$text = "";
		$text_len = ($rgb[0] << 2) + ($rgb[1] << 1) + $rgb[2];
		$key_len = strlen($key);
		$key_index = $color_index = $i = 0;

		while ($i < $text_len)
		{
			# znajd nastpny piksel
			$pixel += ord($key[$key_index]);
			$rgb = $image->get_pixel($pixel);

			# odkoduj dane za pomoc prywatnego klucza
			$byte = chr($rgb[$color_index]) ^ $key[$key_index];
			$text .= $byte;
			
			$i += 1;
			$key_index = ($i % $key_len);
			$color_index = ($i % 3);
		}
		return $text;
	}
}
?>
